# import
from .config import (
    __version__, __author__, __email__, __license__, __description__
)
# app
from .app import vle

__all__ = [
    "__version__",
    "__author__",
    "__email__",
    "__license__",
    "__description__",
    "vle",
]
